/*
 * z2-Environment
 * 
 * Copyright(c) ZFabrik Software GmbH & Co. KG
 * 
 * contact@zfabrik.de
 * 
 * http://www.z2-environment.eu
 */
package com.zfabrik.home.console;

import java.lang.management.ManagementFactory;
import java.util.Date;
import java.util.List;

import javax.management.MBeanServer;
import javax.management.ObjectName;
import javax.management.StandardMBean;

import com.zfabrik.sync.SynchronizationRunner;

public class HomeMBeanImpl implements HomeMBean {

	/**
	 * @param homeLauncher
	 */
	public HomeMBeanImpl() {}

	private Date lastSync = new Date();
	private List<String> lastSyncLog;

	public void synchronize() throws Exception {
		_sync(true,true);
	}

	public void verify() throws Exception {
		_sync(false,true);
	}

	public void averify() throws Exception {
		_sync(false,false);
	}

	public void asynchronize() throws Exception {
		_sync(true,false);
	}

	private void _sync(boolean invalidation, boolean synchronously) {
		final SynchronizationRunner sr = new SynchronizationRunner();
		sr.setMode((invalidation? SynchronizationRunner.INVALIDATE_AND_VERIFY:SynchronizationRunner.VERIFY_ONLY));
		sr.setKeepLog(true);
		sr.setWhenDone(new Runnable() {
			public void run() {
				synchronized (HomeMBeanImpl.this) {
					HomeMBeanImpl.this.lastSyncLog = sr.getLog();
					HomeMBeanImpl.this.lastSync = new Date();
				}
			}
		});
		sr.execute(synchronously);
	}
	
	public void clear() {
		this.lastSyncLog = null;
	}

	public Date getSynchronizationTime() {
		return this.lastSync;
	}

	public List<String> getSynchronizationLog() {
		return this.lastSyncLog;
	}

	public static void register() throws Exception {
		ManagementFactory.getPlatformMBeanServer().registerMBean(
			new StandardMBean(
				new HomeMBeanImpl(), 
				HomeMBean.class, 
				false
			),
			ObjectName.getInstance(HomeMBean.OBJECTNAME)
		);
	}

	public static void unregister() throws Exception {
		MBeanServer mbs = ManagementFactory.getPlatformMBeanServer(); 
		ObjectName on = ObjectName.getInstance(HomeMBean.OBJECTNAME);
		if (mbs.isRegistered(on)) {
			mbs.unregisterMBean(on);
		}
	}

}