/*
 * z2-Environment
 * 
 * Copyright(c) ZFabrik Software GmbH & Co. KG
 * 
 * contact@zfabrik.de
 * 
 * http://www.z2-environment.eu
 */
package com.zfabrik.dev.local;

import com.zfabrik.util.sync.ISynchronization;

/**
 * public interface of dev repo.
 * @author hb
 *
 */

public abstract class DevRepository {
	protected volatile static DevRepository instance;
	
	/**
	 * Property to be specified in the component descriptor to identify the 
	 * associated project for the case of a component level check out.
	 */
	public final static String PROP_PROJECT = "com.zfabrik.dev.project";
	
	public static DevRepository instance() {
		return instance;
	}
	
	/**
	 * retrieve id of a current update, or zero if there was none
	 * @return
	 */
	public abstract long getUpdated();
	/**
	 * should be same update key as retrieved from {@link #getUpdated()}
	 * @param up
	 */
	public abstract void resetUpdated(long up);

	/**
	 * drop current db (and reload on next demand) if older than rev
	 */
	public abstract void releaseDB(long rev);
	
	/**
	 * update db and derive invalidations
	 * @param sync
	 */
	public abstract void preInvalidation(ISynchronization sync);
		
}
