/*
 * z2-Environment
 * 
 * Copyright(c) ZFabrik Software GmbH & Co. KG
 * 
 * contact@zfabrik.de
 * 
 * http://www.z2-environment.eu
 */
package com.zfabrik.impl.dev.transport.gui;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Frame;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.text.DateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.KeyStroke;
import javax.swing.SwingConstants;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.TableRowSorter;

import org.tmatesoft.svn.core.ISVNLogEntryHandler;
import org.tmatesoft.svn.core.SVNException;
import org.tmatesoft.svn.core.SVNLogEntry;
import org.tmatesoft.svn.core.io.SVNRepository;

public class RevPicker  implements ActionListener {
	private static final String AC_CANCEL = "CA";
	private static final String AC_OK = "OK";
	private static final String AC_CHOOSE_HEAD = "HE";
	private static final String AC_DISPLAY = "DI";
	private Destination dest;
	private JTextField sw_range;
	private JTable sw_logs;
	private String range="20";
	private boolean ok;
	private JDialog di;
	private String rev="";
	private List<RevItem> revs = new ArrayList<RevItem>();

	private static class RevItem {
		public static final String[] COLS = new String[]{"rev", "comment","date","author"};  
		
		long rev;
		String comment;
		Date date;
		String author;

		public RevItem(long rev, String comment, Date date, String author) {
			super();
			this.rev = rev;
			this.comment = comment;
			this.date = date;
			this.author = author;
		}
		public Object getColumn(int columnIndex) {
			switch (columnIndex)  {
			case 0: return rev;
			case 1: return comment;
			case 2: return date;
			case 3: return author;
			}
			return null;
		}
		public static Class<?> getColumnClass(int columnIndex) {
			switch (columnIndex)  {
			case 0: return Long.class;
			case 1: return String.class;
			case 2: return Date.class;
			case 3: return String.class;
			}
			return null;
		}
		public static int getColumnCount() {
			return 4;
		}
		public static String getColumnName(int column) {
			return COLS[column];
		}
	}
	
	@SuppressWarnings("serial")
	private class RevTableModel	extends AbstractTableModel {
		public int getColumnCount() {
			return RevItem.getColumnCount();
		}
		public int getRowCount() {
			return revs.size();
		}
		
		public Object getValueAt(int rowIndex, int columnIndex) {
			return revs.get(rowIndex).getColumn(columnIndex);
		}
		
		public Class<?> getColumnClass(int columnIndex) {
			return RevItem.getColumnClass(columnIndex);
		}
		
		public String getColumnName(int column) {
			return RevItem.getColumnName(column);
		}
	}
	
    @SuppressWarnings("serial")
	public boolean run(Destination dest, Component owner) {
    	try	{
    		this.dest = dest;
    		Component f = owner;
    		while ((f!=null) && (!(f instanceof Frame))) f=f.getParent();
    		if (f==null)
    			throw new IllegalStateException("Failed to identify frame!");
    		
	    	this.di = new JDialog((Frame) f,true);
	    	di.setTitle("SVN Revision picker ("+this.dest.getName()+")");
	    	
	    	JPanel mp = new JPanel();
	    	di.add(mp);
	    	BorderLayout bo = new BorderLayout();
	    	mp.setLayout(bo);
			mp.setBorder(BorderFactory.createEmptyBorder(5,10,5,10));

			//
			// top range selection
			//
	    	JPanel bp = new JPanel(); 
	    	mp.add(BorderLayout.NORTH,bp);
	    	
			GridBagLayout grid = new GridBagLayout();
			GridBagConstraints c = new GridBagConstraints();
			c.fill = GridBagConstraints.BOTH;
			c.insets = new Insets(5,2,5,2);
			bp.setLayout(grid);
			bp.setBorder(BorderFactory.createEmptyBorder(0,0,5,0));

			c.weightx=0;
			c.gridwidth=1;
			_addWithConstraints(bp, grid, c, _makeLabel("Log Entries"));
			
			c.weightx=0;
			c.gridwidth=1;
			_addWithConstraints(bp, grid, c, (this.sw_range=new JTextField(this.range,4)));
			// enter in range updates list
			this.sw_range.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					_updateLogList();
				}
			});
	    	
			c.weightx=0;
			c.gridwidth=1;
			JButton bu = new JButton("display");
			bu.addActionListener(this);
			bu.setActionCommand(AC_DISPLAY);
			_addWithConstraints(bp, grid, c, bu);
	
			c.weightx=0.3;
			c.gridwidth=GridBagConstraints.REMAINDER;
			_addWithConstraints(bp, grid, c, new JLabel());
			
			//
			// log list
			//
			RevTableModel m = new RevTableModel();
			this.sw_logs = new JTable(m);
			mp.add(BorderLayout.CENTER,new JScrollPane(this.sw_logs));

			this.sw_logs.setFillsViewportHeight(true);
			// better date time rendering
			this.sw_logs.setDefaultRenderer(Date.class, new DefaultTableCellRenderer() {
		        private DateFormat df = DateFormat.getDateTimeInstance(DateFormat.MEDIUM, DateFormat.MEDIUM); 
		        public void setValue(Object value) {
		            setText((value == null) ? "" : df.format(value));
		        }
			});
			// double-click handling
			this.sw_logs.addMouseListener(new java.awt.event.MouseAdapter() {
				public void mouseClicked(java.awt.event.MouseEvent e) {
					if ( e.getClickCount() == 2 ) {
						ok();
					}
				}
			});	
			// enter in table means ok
			this.sw_logs.registerKeyboardAction(new ActionListener() {
				public void actionPerformed(ActionEvent e) {ok();}
			}, 	KeyStroke.getKeyStroke(KeyEvent.VK_ENTER,0),JComponent.WHEN_FOCUSED);
			
			// tooltip (svn comment)
			this.sw_logs.setDefaultRenderer(String.class, 
					new DefaultTableCellRenderer() {
					    public Component getTableCellRendererComponent(JTable table, Object value,
		                        boolean isSelected, boolean hasFocus, int row, int column) {
					    	Component c = super.getTableCellRendererComponent(table, value, isSelected, hasFocus, row, column);
					    	if (c instanceof JComponent) {
					    		String t = (String) sw_logs.getValueAt(row, column);
					    		if (t!=null && t.length()>0) {
					    			((JComponent)c).setToolTipText(t);
					    		} else {
					    			((JComponent)c).setToolTipText(null);
					    		}
					    	}
					    	return c;
					    }
				
					});
			// sorting
			this.sw_logs.setRowSorter(new TableRowSorter<RevTableModel>(m));
			
			
			//
			// bottom buttons
			//
			JPanel ap = new JPanel(); 
			mp.add(BorderLayout.SOUTH,ap);
			grid = new GridBagLayout();
			ap.setLayout(grid);
			
			ap.setBorder(BorderFactory.createEmptyBorder(0,0,5,0));
	
			c.weightx=0;
			c.gridwidth=1;
			bu = new JButton("\"HEAD\"");
			bu.addActionListener(this);
			bu.setActionCommand(AC_CHOOSE_HEAD);
			_addWithConstraints(ap, grid, c, bu);
	
			c.weightx=1.3;
			c.gridwidth=1;
			_addWithConstraints(ap, grid, c, new JLabel());

			c.weightx=0;
			c.gridwidth=1;
			bu = new JButton("Ok");
			bu.setActionCommand(AC_OK);
			bu.addActionListener(this);
			_addWithConstraints(ap, grid, c, bu);
			
			
			c.weightx=0;
			c.gridwidth=GridBagConstraints.REMAINDER;;
			bu = new JButton("Cancel");
			bu.addActionListener(this);
			bu.setActionCommand(AC_CANCEL);
			_addWithConstraints(ap, grid, c, bu);
			
			di.pack();
	        di.setLocationRelativeTo(owner);
	        
	        // 
	        // esc to close and cancel
	        di.getRootPane().registerKeyboardAction(
	        		new ActionListener(){
						public void actionPerformed(ActionEvent e) {
							RevPicker.this.cancel();
						}
					}, 
	        		KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0),
	        		JComponent.WHEN_IN_FOCUSED_WINDOW
	        );
	        
	        // that makes it wait...
	        di.setVisible(true);
	        return ok;
    	} finally {
    		_getRange();
    		this.di = null;
    		this.sw_logs=null;
    		this.dest=null;
    		this.revs.clear();
    	}
    }
	
	private JLabel _makeLabel(String title) {
		JLabel l=new JLabel(title);
		l.setHorizontalAlignment(SwingConstants.RIGHT);
		return l;
	}

	private void _addWithConstraints(JPanel p,GridBagLayout grid,
			GridBagConstraints c, Component comp) {
		grid.setConstraints(comp, c);
		p.add(comp);
	}

	public void actionPerformed(ActionEvent e) {
		String actionCommand = e.getActionCommand();
		if (AC_OK.equals(actionCommand)) {	ok(); } 
		else
		if (AC_CANCEL.equals(actionCommand)) {cancel();} 
		else
		if (AC_CHOOSE_HEAD.equals(actionCommand)) {
			this.rev="HEAD";
			this.ok = true;
			this.di.setVisible(false);
		} else 
		if (AC_DISPLAY.equals(actionCommand)) {
			_updateLogList();
		}
	}

	private String _getRange() {
		if (this.sw_range!=null)
			return this.range = this.sw_range.getText().trim();
		return null;
	}
	
	private void _updateLogList() {
		try {
			int d;
			try {
				d = Integer.parseInt(_getRange());
			} catch (NumberFormatException nfe) {
				TransportTool.error(this.di.getRootPane(), _getRange()+" is not a number!",null);
				return;
			}
			if (d<0) {
				TransportTool.error(this.di.getRootPane(), "Please specify a positive revision range",null);
				return;
			}
			SVNRepository repo = TransportTool.getRepository(this.dest);
			String rp = repo.getRepositoryPath("");
			long lr = repo.getLatestRevision();
			this.revs = new ArrayList<RevItem>(d);
			repo.log(new String[] { rp },
					lr,0,false,false,d,new ISVNLogEntryHandler() {
					public void handleLogEntry(SVNLogEntry le) throws SVNException {
						revs.add(new RevItem(le.getRevision(),le.getMessage(),le.getDate(),le.getAuthor()));
					}
				});
			
			((RevTableModel) this.sw_logs.getModel()).fireTableDataChanged();
		} catch (Exception e) {
			TransportTool.error(this.di.getRootPane(), "Failed to read subverion log", e);
			logger.log(Level.WARNING,"Failed to read subverion log",e);
		}
		
	}

	public String getRev() {
		return this.rev;
	}
	
	public void cancel() {
		if (this.di!=null) {
			this.ok = false;
			this.di.setVisible(false);
		}
	}

	public void ok() {
		if (this.di!=null) {
			int index = this.sw_logs.getSelectedRow();
			if (index>=0) {
				index = sw_logs.convertRowIndexToModel(index);
				rev = Long.toString(revs.get(index).rev);
				this.ok = true;
			} else {
				this.ok = false;
			}
			this.di.setVisible(false);
		}
	}

	private final static Logger logger = Logger.getLogger(RevPicker.class.getName());
}
